-module(icfp).

%% Time (ms) reserved for "oh shit! we're out of time! panic!" Being
%% generous since time is not a big problem
-define(PANIC_TIME, 10000).

-export([file/2,tfile/2,check/2,cmdline/1]).

-include("icfp.hrl").

file(Infile, Outfile) ->
    Istate = #s{},				%This may be explicitly set
    Parse0 = icfp_read:file(Infile, Istate),
    Parse1 = icfp_opt:parse(Parse0, Istate),
    icfp_print:file(Outfile, Parse1, Istate).

tfile(Infile, Outfile) ->
    Istate = #s{},				%This may be explicitly set
    Parse0 = ?TIME("read", icfp_read:file(Infile, Istate)),
    Parse1 = ?TIME("Opt", icfp_opt:parse(Parse0, Istate)),
    ?TIME("Print", icfp_print:file(Outfile, Parse1, Istate)).

check(Ifile, Ofile) ->
    file(Ifile, Ofile),
    {ok,Old} = file:read_file(Ifile),
    {ok,New} = file:read_file(Ofile),
    icfp_webtest:equiv(binary_to_list(Old), binary_to_list(New)).

cmdline([Infile, Outfile, Flagfile]) ->
    start_watchdog(),
    ok = file(atom_to_list(Infile), atom_to_list(Outfile)),
    ok = file:write_file(Flagfile, "ok\n"),
    halt().

%% Set a timer to halt the emulator if we approach the time limit.
start_watchdog() ->
    case os:getenv("TLIMIT") of
	false ->
	    io:format("No TLIMIT env - skipping watchdog.~n"),
	    skip;
	Tstr ->
	    Time = list_to_integer(Tstr) * 1000,
	    timer:start(),
	    timer:apply_after(Time - ?PANIC_TIME, erlang, halt, [])
    end.

