-module(icfp_opt).

-compile(export_all).

-export([parse/2]).

-import(lists,[reverse/1]).

-include("icfp.hrl").

%% parse([Parse], PrevState) -> [Parse].
%% parse([Parse], PrevChar, PrevState) -> [Parse].
%%  Optimise the parse.  First trim away unnecessary spaces.

parse(P, Ps) -> trim_spaces(P, false, Ps).

trim_spaces([{S,Cs0}|Parse], Pc0, Pstate) ->
    {Cs1,Pc1} = trim_spaces(Cs0, Pc0, Pstate, S),
    %% This can result in empty states which we remove.
    if  Cs1 == << >> -> trim_spaces(Parse, Pc1, S);
        true -> [{S,Cs1}|trim_spaces(Parse, Pc1, S)]
    end;
trim_spaces([], Pc, Ps) -> [].

trim_spaces(Cs0, Pc0, Pstate, Cstate) ->
    {Cs1,Pc1} = trim_spaces(binary_to_list(Cs0), Pc0, Pstate, Cstate, []),
    {list_to_binary(Cs1),Pc1}.

trim_spaces([C|Cs], true, Pstate, Cstate, L) when
  C == $\s; C == $\t; C == $\n; C == $\r ->
    case equal_space_context(Pstate, Cstate) of
	true -> trim_spaces(Cs, true, Pstate, Cstate, L);
	false -> trim_spaces(Cs, true, Cstate, Cstate, [C|L])
    end;
trim_spaces([C|Cs], Pc, Pstate, Cstate, L) ->
    trim_spaces(Cs, is_white_space(C), Cstate, Cstate, [C|L]);
trim_spaces([], Pc, Pstate, Cstate, L) -> {reverse(L),Pc}.

%% equal_space_context(OldState, NewState) -> true | false.
%%  Check if the two states are equal for the purpose of space
%%  contexts.  This is straight out of the Specification!

equal_space_context(Old, #s{tt=false}=New) ->
    make_space_context(Old) == make_space_context(New);
equal_space_context(Old, New) -> false.

make_space_context(#s{u=U}=S0) ->
    Color = if U == 0 -> w;
	       true -> S0#s.color
	    end,
    S0#s{b=false,em=false,i=false,s=false,color=Color}.

is_white_space(C) when C == $\s; C == $\t; C == $\n; C == $\r -> true;
is_white_space(C) -> false.
